import unittest
from StringIO import StringIO
from lxml import etree
import stix.bindings as bindings
import stix.utils.parser
from stix.test import EntityTestCase
from stix.extensions.malware.maec_4_1_malware import MAECInstance

class PythonMAECTests(EntityTestCase, unittest.TestCase):
    klass = MAECInstance

    _full_dict = {
        'xsi:type': 'stix-maec:MAEC4.1InstanceType',
        'maec': {
            'malware_subjects':
                [
                    {'malware_instance_object_attributes':
                         {'id': 'maec-tst-obj-1',
                          'properties': {
                              'hashes':
                                  [
                                      {
                                          'simple_hash_value': '9d7006e30fdf15e9c8e03e62534b3a3e',
                                          'type': 'MD5'
                                      }
                                  ],
                              'xsi:type': 'FileObjectType'}
                         }
                    }
                ]
        }
    }

class PythonMAECEtreeTests(unittest.TestCase):
    XML = (
        """
        <stix-maec:MAEC
            xmlns:cyboxCommon="http://cybox.mitre.org/common-2"
            xmlns:cybox="http://cybox.mitre.org/cybox-2"
            xmlns:cyboxVocabs="http://cybox.mitre.org/default_vocabularies-2"
            xmlns:FileObj="http://cybox.mitre.org/objects#FileObject-2"
            xmlns:example="http://example.com"
            xmlns:maecPackage="http://maec.mitre.org/XMLSchema/maec-package-2"
            xmlns:maecVocabs="http://maec.mitre.org/default_vocabularies-1"
            xmlns:ttp="http://stix.mitre.org/TTP-1"
            xmlns:stixCommon="http://stix.mitre.org/common-1"
            xmlns:stixVocabs="http://stix.mitre.org/default_vocabularies-1"
            xmlns:stix-maec="http://stix.mitre.org/extensions/Malware#MAEC4.1-1"
            xmlns:stix="http://stix.mitre.org/stix-1"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="
            http://cybox.mitre.org/common-2 http://cybox.mitre.org/XMLSchema/common/2.1/cybox_common.xsd
            http://cybox.mitre.org/cybox-2 http://cybox.mitre.org/XMLSchema/core/2.1/cybox_core.xsd
            http://cybox.mitre.org/default_vocabularies-2 http://cybox.mitre.org/XMLSchema/default_vocabularies/2.1/cybox_default_vocabularies.xsd
            http://cybox.mitre.org/objects#FileObject-2 http://cybox.mitre.org/XMLSchema/objects/File/2.1/File_Object.xsd
            http://maec.mitre.org/XMLSchema/maec-package-2 http://maec.mitre.org/language/version4.1/maec_package_schema.xsd
            http://maec.mitre.org/default_vocabularies-1 http://maec.mitre.org/language/version4.1/maec_default_vocabularies.xsd
            http://stix.mitre.org/TTP-1 http://stix.mitre.org/XMLSchema/ttp/1.1.1/ttp.xsd
            http://stix.mitre.org/common-1 http://stix.mitre.org/XMLSchema/common/1.1.1/stix_common.xsd
            http://stix.mitre.org/default_vocabularies-1 http://stix.mitre.org/XMLSchema/default_vocabularies/1.1.1/stix_default_vocabularies.xsd
            http://stix.mitre.org/extensions/Malware#MAEC4.1-1 http://stix.mitre.org/XMLSchema/extensions/malware/maec_4.1/1.0.1/maec_4.1_malware.xsd
            http://stix.mitre.org/stix-1 http://stix.mitre.org/XMLSchema/core/1.1.1/stix_core.xsd" xsi:type='stix-maec:MAEC4.1InstanceType'
            id="example:package-2fb96bef-1b11-436e-af4a-15588ac3198b" schema_version="2.1">
            <maecPackage:Malware_Subjects>
                <maecPackage:Malware_Subject>
                    <maecPackage:Malware_Instance_Object_Attributes id="maec-tst-obj-1">
                        <cybox:Properties xsi:type="FileObj:FileObjectType">
                            <FileObj:Hashes>
                                <cyboxCommon:Hash>
                                    <cyboxCommon:Type xsi:type="cyboxVocabs:HashNameVocab-1.0">MD5</cyboxCommon:Type>
                                    <cyboxCommon:Simple_Hash_Value>9d7006e30fdf15e9c8e03e62534b3a3e</cyboxCommon:Simple_Hash_Value>
                                </cyboxCommon:Hash>
                            </FileObj:Hashes>
                        </cybox:Properties>
                    </maecPackage:Malware_Instance_Object_Attributes>
                </maecPackage:Malware_Subject>
            </maecPackage:Malware_Subjects>
        </stix-maec:MAEC>
        """
    )

    def _test_xml(self, obj):
        xml = obj.to_xml()
        parser = stix.utils.parser.get_xml_parser()
        tree = etree.parse(StringIO(xml), parser=parser)
        root = tree.getroot()

        xpath = "//cyboxCommon:Type"
        nodes = root.xpath(xpath, namespaces={'cyboxCommon': 'http://cybox.mitre.org/common-2'})

        self.assertTrue(nodes != None)
        self.assertEqual(len(nodes), 1)
        self.assertEqual(nodes[0].text, "MD5")


    def test_etree(self):
        parser = stix.utils.parser.get_xml_parser()
        tree = etree.parse(StringIO(self.XML), parser=parser)

        ext = MAECInstance()
        ext.maec = tree
        self._test_xml(ext)


    def test_etree_dict(self):
        parser = stix.utils.parser.get_xml_parser()
        tree = etree.parse(StringIO(self.XML), parser=parser)
        ext = MAECInstance()
        ext.maec = tree

        d = ext.to_dict()
        ext2 = MAECInstance.from_dict(d)
        self._test_xml(ext2)


if __name__ == "__main__":
    unittest.main()


import unittest

from stix.test import EntityTestCase

from stix.common.related import RelatedPackageRef


class StatementTests(EntityTestCase, unittest.TestCase):
    klass = RelatedPackageRef
    _full_dict = {
        'idref': "example:Campaign-133",
        'timestamp': "2014-01-31T06:14:46",
        'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
        'information_source': {
            'description': "Source of the relationship",
        },
        'relationship': "Associated",
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.utils
from stix.utils import dates
from stix.common import (GenericRelationshipList, RelatedObservable, StructuredText,
                         Confidence, InformationSource)
import stix.bindings.indicator as indicator_binding
from datetime import datetime
from dateutil.tz import tzutc

class Sighting(stix.Entity):
    _namespace = "http://stix.mitre.org/Indicator-2"
    _binding = indicator_binding
    _binding_class = _binding.SightingType
    
    def __init__(self, timestamp=None, timestamp_precision=None, description=None):
        self.timestamp = timestamp or datetime.now(tzutc())
        self.timestamp_precision = timestamp_precision
        self.description = description
        self.source = None
        self.reference = None
        self.confidence = None
        self.related_observables = RelatedObservables()
        
    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = dates.parse_value(value)
    
    @property
    def description(self):
        return self._description
    
    @description.setter
    def description(self, value):
        if not value:
            self._description = None
        elif isinstance(value, StructuredText):
            self._description = value
        else:
            self._description = StructuredText(value)
            
    @property
    def source(self):
        return self._source
    
    @source.setter
    def source(self, value):
        if value is None:
            self._source = None
        elif isinstance(value, InformationSource):
            self._source = value
        else:
            raise ValueError("source must be of type InformationSource")
          
    
    @property
    def confidence(self):
        return self._confidence
    
    @confidence.setter
    def confidence(self, value):
        if not value:
            self._confidence = None
        elif isinstance(value, Confidence):
            self._confidence = value
        else:
            self._confidence = Confidence(value)
    
    def to_obj(self, return_obj=None, ns_info=None):
        super(Sighting, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()
        
        return_obj.timestamp = dates.serialize_value(self.timestamp)
        return_obj.timestamp_precision = self.timestamp_precision
        return_obj.Reference = self.reference
        
        if self.source:
            return_obj.Source = self.source.to_obj(ns_info=ns_info)
        if self.confidence:
            return_obj.Confidence = self.confidence.to_obj(ns_info=ns_info)
        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.related_observables:
            return_obj.Related_Observables = self.related_observables.to_obj(ns_info=ns_info)
        
        return return_obj

    def to_dict(self):
        d = {}
        if self.timestamp:
            d['timestamp'] = dates.serialize_value(self.timestamp)
        if self.timestamp_precision:
            d['timestamp_precision'] = str(self.timestamp_precision)
        if self.source:
            d['source'] = self.source.to_dict()
        if self.reference:
            d['reference'] = str(self.reference)
        if self.confidence:
            d['confidence'] = self.confidence.to_dict()
        if self.description:
            d['description'] = self.description.to_dict()
        if self.related_observables:
            d['related_observables'] = self.related_observables.to_dict()
         
        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if return_obj is None:
            return_obj = cls()
        
        return_obj.timestamp = obj.timestamp
        return_obj.timestamp_precision = obj.timestamp_precision
        return_obj.source = InformationSource.from_obj(obj.Source)
        return_obj.refernce = obj.Reference
        return_obj.confidence = Confidence.from_obj(obj.Confidence)
        return_obj.description = StructuredText.from_obj(obj.Description)
        return_obj.related_observables = RelatedObservables.from_obj(obj.Related_Observables)
        return return_obj

    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if return_obj is None:
            return_obj = cls()

        return_obj.timestamp = d.get('timestamp')
        return_obj.timestamp_precision = d.get('timestamp_precision')
        return_obj.source = InformationSource.from_dict(d.get('source'))
        return_obj.reference = d.get('reference')
        return_obj.confidence = Confidence.from_dict(d.get('confidence'))
        return_obj.description = StructuredText.from_dict(d.get('description'))
        return_obj.related_observables = RelatedObservables.from_dict(d.get('related_observables'))
        return return_obj

class Sightings(stix.EntityList):
    _namespace = "http://stix.mitre.org/Indicator-2"
    _binding = indicator_binding
    _binding_class = _binding.SightingsType
    _contained_type = Sighting
    _binding_var = "Sighting"
    _inner_name = "sightings"
    
    def __init__(self, sightings_count=None, *args):
        super(Sightings, self).__init__(*args)
        self.sightings_count = sightings_count

    def __nonzero__(self):
        return super(Sightings, self).__nonzero__() or bool(self.sightings_count)

    def to_obj(self, return_obj=None, ns_info=None):
        list_obj = super(Sightings, self).to_obj(return_obj=return_obj, ns_info=ns_info)
        list_obj.sightings_count = self.sightings_count
        return list_obj

    def to_dict(self):
        d = super(Sightings, self).to_dict()
        if self.sightings_count:
            d['sightings_count'] = self.sightings_count
        return d

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if return_obj is None:
            return_obj = cls()

        super(Sightings, cls).from_obj(obj, return_obj=return_obj)
        return_obj.sightings_count = obj.sightings_count
        return return_obj

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if return_obj is None:
            return_obj = cls()

        super(Sightings, cls).from_dict(dict_repr, return_obj=return_obj)
        return_obj.sightings_count = dict_repr.get('sightings_count')
        return return_obj

class RelatedObservables(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/Indicator-2"
    _binding = indicator_binding
    _binding_class = _binding.RelatedObservablesType
    _binding_var = "Related_Observable"
    _contained_type = RelatedObservable
    _inner_name = "observables"



import unittest
from stix.test import EntityTestCase
from stix.coa import CourseOfAction

class COATests(EntityTestCase, unittest.TestCase):
    klass = CourseOfAction
    _full_dict = {
        'id': 'example:coa-1',
        'timestamp': "2014-03-20T04:35:12",
        'version': '1.1',
        'title': "COA1",
        'stage':  {'value': 'Remedy', 'xsi:type': 'stixVocabs:COAStageVocab-1.0'},
        'type': {'value': 'Redirection', 'xsi:type': 'stixVocabs:CourseOfActionTypeVocab-1.0'},
        'description': "This is a long description about a course of action",
        'short_description': "a COA",
        'objective': {
            'description': "This is why we're taking this action",
            'short_description': "Stop the bad stuff",
            'applicability_confidence': {'value': {'value': 'Medium', 
                                                   'xsi:type': 'stixVocabs:HighMediumLowVocab-1.0'}},

        },
        'parameter_observables': {
            'major_version': 2,
            'minor_version': 1,
            'update_version': 0,
            'observables': [
                {
                    'idref': "example:Observable-1"
                }
            ]
        },
        'impact': {'value': {'value': 'Medium', 
                             'xsi:type': 'stixVocabs:HighMediumLowVocab-1.0'}},
        'cost': {'value': {'value': 'Medium', 
                           'xsi:type': 'stixVocabs:HighMediumLowVocab-1.0'}},
        'efficacy': {'value': {'value': 'Medium', 
                               'xsi:type': 'stixVocabs:HighMediumLowVocab-1.0'}},
        'information_source': {
            'description': "Mr. Evil's enemy",
            'identity': {
                'name': "Ms. Good",
            },
        },
        'handling': [
            {
                'marking_structures': [{
                    'marking_model_name': 'TLP',
                    'color': "GREEN",
                    'xsi:type': "tlpMarking:TLPMarkingStructureType",
                }]
            }
        ],
        'related_coas': {
            'scope': "exclusive",
            'coas': [
                {
                    'confidence': {'value': {'value': "Medium", 'xsi:type':'stixVocabs:HighMediumLowVocab-1.0'}},
                    'course_of_action': {'idref': "example:COA-52",
                                         'version': '1.1.1'},
                }
            ]
        },
        'related_packages': {
            'packages': [
                {'idref': "example:Package-AB", 'relationship': "Parent"},
                {'idref': "example:Package-CD", 'relationship': "Child"}
            ]
        },
    }

if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.utils
from stix.utils import dates
import stix.bindings.ttp as ttp_binding
from stix.common import StructuredText, VocabString, InformationSource, Statement
from stix.common.vocabs import IntendedEffect
from stix.data_marking import Marking
from .behavior import Behavior
from .resource import Resource
from .victim_targeting import VictimTargeting
from datetime import datetime
from dateutil.tz import tzutc

class TTP(stix.Entity):
    _binding = ttp_binding
    _binding_class = _binding.TTPType
    _namespace = "http://stix.mitre.org/TTP-1"
    _version = "1.1.1"
    _ALL_VERSIONS = ("1.0", "1.0.1", "1.1", "1.1.1")

    def __init__(self, id_=None, idref=None, timestamp=None, title=None, description=None, short_description=None):
        self.id_ = id_ or stix.utils.create_id("ttp")
        self.idref = idref
        self.version = None # self._version
        self.title = title
        self.description = description
        self.short_description = short_description
        self.behavior = None
        self.related_ttps = None
        self.information_source = None
        self.intended_effects = None
        self.resources = None
        self.victim_targeting = None
        self.handling = None
        self.exploit_targets = ExploitTargets()

        if timestamp:
            self.timestamp = timestamp
        else:
            self.timestamp = datetime.now(tzutc()) if not idref else None

    @property
    def id_(self):
        return self._id
    
    @id_.setter
    def id_(self, value):
        if not value:
            self._id = None
        else:
            self._id = value
            self.idref = None
    
    @property
    def version(self):
        return self._version
    
    @version.setter
    def version(self, value):
        if not value:
            self._version = None
        else:
            if value not in self._ALL_VERSIONS:
                msg = ("Version must be one of %s. Found '%s'" %
                      (self._ALL_VERSIONS, value))
                raise ValueError(msg)
            self._version = value
    
    @property
    def idref(self):
        return self._idref
    
    @idref.setter
    def idref(self, value):
        if not value:
            self._idref = None
        else:
            self._idref = value
            self.id_ = None # unset id_ if idref is present
    
    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = dates.parse_value(value)

    @property
    def title(self):
        return self._title

    @title.setter
    def title(self, value):
        self._title = value

    @property
    def description(self):
        return self._description

    @description.setter
    def description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._description = value
            else:
                self._description = StructuredText(value=value)
        else:
            self._description = None

    @property
    def short_description(self):
        return self._short_description

    @short_description.setter
    def short_description(self, value):
        if value:
            if isinstance(value, StructuredText):
                self._short_description = value
            else:
                self._short_description = StructuredText(value=value)
        else:
            self._short_description = None

    @property
    def behavior(self):
        return self._behavior

    @behavior.setter
    def behavior(self, value):
        if not value:
            self._behavior = None
        elif isinstance(value, Behavior):
            self._behavior = value
        else:
            raise ValueError('Value must be a Behavior instance')

    @property
    def related_ttps(self):
        return self._related_ttps

    @related_ttps.setter
    def related_ttps(self, value):

        if not value:
            self._related_ttps = RelatedTTPs()
        elif isinstance(value, RelatedTTPs):
            self._related_ttps = value
        else:
            raise ValueError("value must be RelatedTTPs instance")

    @property
    def exploit_targets(self):
        return self._exploit_targets

    @exploit_targets.setter
    def exploit_targets(self, value):
        if not value:
            self._exploit_targets = ExploitTargets()
        elif isinstance(value, ExploitTargets):
            self._exploit_targets = value
        else:
            raise ValueError("value must be ExploitTargets instance")

    @property
    def information_source(self):
        return self._information_source

    @information_source.setter
    def information_source(self, value):
        if not value:
            self._information_source = None
        elif isinstance(value, InformationSource):
            self._information_source = value
        else:
            raise ValueError('value must be instance of InformationSource')

    @property
    def intended_effects(self):
        return self._intended_effects

    @intended_effects.setter
    def intended_effects(self, value):
        self._intended_effects = []

        if not value:
            return
        elif isinstance(value, list):
            for v in value:
                self.add_intended_effect(v)
        else:
            self.add_intended_effect(value)

    def add_intended_effect(self, value):
        if not value:
            return
        elif isinstance(value, Statement):
            self.intended_effects.append(value)
        else:
            intended_effect = IntendedEffect(value)
            self.intended_effects.append(Statement(value=intended_effect))

    @property
    def resources(self):
        return self._resources

    @resources.setter
    def resources(self, value):
        if not value:
            self._resources = None
        elif isinstance(value, Resource):
            self._resources = value
        else:
            raise ValueError('value must be instance of Resource')

    @property
    def victim_targeting(self):
        return self._victim_targeting

    @victim_targeting.setter
    def victim_targeting(self, value):
        if not value:
            self._victim_targeting = None
        elif isinstance(value, VictimTargeting):
            self._victim_targeting = value
        else:
            raise ValueError('value must be instance of VictimTargeting')

    @property
    def handling(self):
        return self._handling

    @handling.setter
    def handling(self, value):
        if value and not isinstance(value, Marking):
            raise ValueError('value must be instance of Marking')

        self._handling = value

    def to_obj(self, return_obj=None, ns_info=None):
        super(TTP, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.id = self.id_
        return_obj.idref = self.idref
        return_obj.timestamp = dates.serialize_value(self.timestamp)
        return_obj.version = self.version
        return_obj.Title = self.title

        if self.description:
            return_obj.Description = self.description.to_obj(ns_info=ns_info)
        if self.short_description:
            return_obj.Short_Description = self.short_description.to_obj(ns_info=ns_info)
        if self.behavior:
            return_obj.Behavior = self.behavior.to_obj(ns_info=ns_info)
        if self.related_ttps:
            return_obj.Related_TTPs = self.related_ttps.to_obj(ns_info=ns_info)
        if self.exploit_targets:
            return_obj.Exploit_Targets = self.exploit_targets.to_obj(ns_info=ns_info)
        if self.information_source:
            return_obj.Information_Source = self.information_source.to_obj(ns_info=ns_info)
        if self.intended_effects:
            return_obj.Intended_Effect = [x.to_obj(ns_info=ns_info) for x in self.intended_effects]
        if self.resources:
            return_obj.Resources = self.resources.to_obj(ns_info=ns_info)
        if self.victim_targeting:
            return_obj.Victim_Targeting = self.victim_targeting.to_obj(ns_info=ns_info)
        if self.handling:
            return_obj.Handling = self.handling.to_obj(ns_info=ns_info)

        return return_obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = obj.id
        return_obj.idref = obj.idref
        return_obj.timestamp = obj.timestamp

        if isinstance(obj, cls._binding_class): # TTPType properties
            return_obj.version = obj.version
            return_obj.title = obj.Title
            return_obj.description = StructuredText.from_obj(obj.Description)
            return_obj.short_description = StructuredText.from_obj(obj.Short_Description)
            return_obj.behavior = Behavior.from_obj(obj.Behavior)
            return_obj.related_ttps = RelatedTTPs.from_obj(obj.Related_TTPs)
            return_obj.exploit_targets = ExploitTargets.from_obj(obj.Exploit_Targets)
            return_obj.information_source = InformationSource.from_obj(obj.Information_Source)
            return_obj.resources = Resource.from_obj(obj.Resources)
            return_obj.victim_targeting = VictimTargeting.from_obj(obj.Victim_Targeting)
            return_obj.handling = Marking.from_obj(obj.Handling)

            if obj.Intended_Effect:
                return_obj.intended_effects = [Statement.from_obj(x) for x in obj.Intended_Effect]

        return return_obj

    def to_dict(self):
        d = {}
        if self.id_:
            d['id'] = self.id_
        if self.idref:
            d['idref'] = self.idref
        if self.timestamp:
            d['timestamp'] = dates.serialize_value(self.timestamp)
        if self.version:
            d['version'] = self.version
        if self.title:
            d['title'] = self.title
        if self.description:
            d['description'] = self.description.to_dict()
        if self.short_description:
            d['short_description'] = self.short_description.to_dict()
        if self.behavior:
            d['behavior'] = self.behavior.to_dict()
        if self.related_ttps:
            d['related_ttps'] = self.related_ttps.to_dict()
        if self.exploit_targets:
            d['exploit_targets'] = self.exploit_targets.to_dict()
        if self.information_source:
            d['information_source'] = self.information_source.to_dict()
        if self.intended_effects:
            d['intended_effects'] = [x.to_dict() for x in self.intended_effects]
        if self.resources:
            d['resources'] = self.resources.to_dict()
        if self.victim_targeting:
            d['victim_targeting'] = self.victim_targeting.to_dict()
        if self.handling:
            d['handling'] = self.handling.to_dict()

        return d

    @classmethod
    def from_dict(cls, dict_repr, return_obj=None):
        if not dict_repr:
            return None
        if not return_obj:
            return_obj = cls()

        return_obj.id_ = dict_repr.get('id')
        return_obj.idref = dict_repr.get('idref')
        return_obj.timestamp = dict_repr.get('timestamp')
        return_obj.version = dict_repr.get('version')
        return_obj.title = dict_repr.get('title')
        return_obj.description = StructuredText.from_dict(dict_repr.get('description'))
        return_obj.short_description = StructuredText.from_dict(dict_repr.get('short_description'))
        return_obj.behavior = Behavior.from_dict(dict_repr.get('behavior'))
        return_obj.related_ttps = RelatedTTPs.from_dict(dict_repr.get('related_ttps'))
        return_obj.exploit_targets = ExploitTargets.from_dict(dict_repr.get('exploit_targets'))
        return_obj.information_source = InformationSource.from_dict(dict_repr.get('information_source'))
        return_obj.intended_effects = [Statement.from_dict(x) for x in dict_repr.get('intended_effects', [])]
        return_obj.resources = Resource.from_dict(dict_repr.get('resources'))
        return_obj.victim_targeting = VictimTargeting.from_dict(dict_repr.get('victim_targeting'))
        return_obj.handling = Marking.from_dict(dict_repr.get('handling'))

        return return_obj

# Avoid circular imports
from .related_ttps import RelatedTTPs
from .exploit_targets import ExploitTargets



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# Generated Thu Apr 11 15:07:47 2013 by generateDS.py version 2.9a.
#

import sys
from stix.bindings import *
import stix.bindings.ttp as ttp_binding
try:
    from maec.bindings.maec_package import PackageType
    maec_installed = True
except ImportError:
    maec_installed = False

XML_NS = "http://stix.mitre.org/extensions/Malware#MAEC4.1-1"

#
# Data representation classes.
#

class MAEC4_1InstanceType(ttp_binding.MalwareInstanceType):
    """The MAEC4.1InstanceType provides an extension to ttp_binding.MalwareInstanceType
    which imports and leverages the MAEC 4.0.1 schema for structured
    characterization of Malware."""
    subclass = None
    superclass = ttp_binding.MalwareInstanceType
    def __init__(self, Type=None, Name=None, Description=None, MAEC=None):
        super(MAEC4_1InstanceType, self).__init__(Type=Type, Name=Name, Description=Description)
        self.xmlns          = XML_NS
        self.xmlns_prefix   = "stix-maec"
        self.xml_type       = "MAEC4.1InstanceType"
        self.MAEC = MAEC
    def factory(*args_, **kwargs_):
        if MAEC4_1InstanceType.subclass:
            return MAEC4_1InstanceType.subclass(*args_, **kwargs_)
        else:
            return MAEC4_1InstanceType(*args_, **kwargs_)
    factory = staticmethod(factory)
    def get_MAEC(self): return self.MAEC
    def set_MAEC(self, MAEC): self.MAEC = MAEC
    def hasContent_(self):
        if (
            self.MAEC is not None or
            super(MAEC4_1InstanceType, self).hasContent_()
            ):
            return True
        else:
            return False
    def export(self, lwrite, level, nsmap, namespace_=XML_NS, name_='MAEC4.1InstanceType', namespacedef_='', pretty_print=True):
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        showIndent(lwrite, level, pretty_print)
        lwrite('<%s:%s%s' % (nsmap[namespace_], name_, namespacedef_ and ' ' + namespacedef_ or '', ))
        already_processed = set()
        self.exportAttributes(lwrite, level, already_processed, namespace_, name_='MAEC4.1InstanceType')
        if self.hasContent_():
            lwrite('>%s' % (eol_, ))
            self.exportChildren(lwrite, level + 1, nsmap, XML_NS, name_, pretty_print=pretty_print)
            showIndent(lwrite, level, pretty_print)
            lwrite('</%s:%s>%s' % (nsmap[namespace_], name_, eol_))
        else:
            lwrite('/>%s' % (eol_, ))
    def exportAttributes(self, lwrite, level, already_processed, namespace_='', name_='MAEC4.1InstanceType'):
        super(MAEC4_1InstanceType, self).exportAttributes(lwrite, level, already_processed, namespace_, name_='MAEC4.1InstanceType')
        if 'xsi:type' not in already_processed:
            already_processed.add('xsi:type')
            xsi_type = " xsi:type='%s:%s'" % (self.xmlns_prefix, self.xml_type)
            lwrite(xsi_type)
    def exportChildren(self, lwrite, level, nsmap, namespace_=XML_NS, name_='MAEC4.1InstanceType', fromsubclass_=False, pretty_print=True):
        super(MAEC4_1InstanceType, self).exportChildren(lwrite, level, nsmap, ttp_binding.XML_NS, name_, True, pretty_print=pretty_print)
        if pretty_print:
            eol_ = '\n'
        else:
            eol_ = ''
        if self.MAEC is not None:
            if maec_installed and isinstance(self.MAEC, PackageType):
                self.MAEC.export(lwrite, level, namespace_='stix-maec:', name_='MAEC', pretty_print=pretty_print)
            else:
                showIndent(lwrite, level, pretty_print)
                lwrite(etree_.tostring(self.MAEC, pretty_print=pretty_print))
    def build(self, node):
        already_processed = set()
        self.buildAttributes(node, node.attrib, already_processed)
        for child in node:
            nodeName_ = Tag_pattern_.match(child.tag).groups()[-1]
            self.buildChildren(child, node, nodeName_)
    def buildAttributes(self, node, attrs, already_processed):
        super(MAEC4_1InstanceType, self).buildAttributes(node, attrs, already_processed)
    def buildChildren(self, child_, node, nodeName_, fromsubclass_=False):
        if nodeName_ == 'MAEC':
            if maec_installed:
                obj_ = PackageType.factory()
                obj_.build(child_)
                self.set_MAEC(obj_)
            else:
                self.set_MAEC(child_)
        super(MAEC4_1InstanceType, self).buildChildren(child_, node, nodeName_, True)
# end class MAEC4_1InstanceType

GDSClassesMapping = {}

USAGE_TEXT = """
Usage: python <Parser>.py [ -s ] <in_xml_file>
"""

def usage():
    print USAGE_TEXT
    sys.exit(1)

def get_root_tag(node):
    tag = Tag_pattern_.match(node.tag).groups()[-1]
    rootClass = GDSClassesMapping.get(tag)
    if rootClass is None:
        rootClass = globals().get(tag)
    return tag, rootClass

def parse(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'MAEC4.1InstanceType'
        rootClass = MAEC4_1InstanceType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    #sys.stdout.write('<?xml version="1.0" ?>\n')
    #rootObj.export(sys.stdout, 0, name_=rootTag,
    #    namespacedef_='',
    #    pretty_print=True)
    return rootObj

def parseEtree(inFileName):
    doc = parsexml_(inFileName)
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'MAEC4.1InstanceType'
        rootClass = MAEC4_1InstanceType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    rootElement = rootObj.to_etree(None, name_=rootTag)
    content = etree_.tostring(rootElement, pretty_print=True,
        xml_declaration=True, encoding="utf-8")
    sys.stdout.write(content)
    sys.stdout.write('\n')
    return rootObj, rootElement

def parseString(inString):
    from StringIO import StringIO
    doc = parsexml_(StringIO(inString))
    rootNode = doc.getroot()
    rootTag, rootClass = get_root_tag(rootNode)
    if rootClass is None:
        rootTag = 'MAEC4.1InstanceType'
        rootClass = MAEC4_1InstanceType
    rootObj = rootClass.factory()
    rootObj.build(rootNode)
    # Enable Python to collect the space used by the DOM.
    doc = None
    #sys.stdout.write('<?xml version="1.0" ?>\n')
    #rootObj.export(lwrite=sys.stdout, level=0, name_="MAEC4.1InstanceType",
    #    namespacedef_='')
    return rootObj

def main():
    args = sys.argv[1:]
    if len(args) == 1:
        parse(args[0])
    else:
        usage()

if __name__ == '__main__':
    #import pdb; pdb.set_trace()
    main()

__all__ = [
    "MAEC4_1InstanceType"
    ]



import stix

from stix.bindings.data_marking import MarkingSpecificationType, MarkingStructureType, MarkingType
from stix.common import StructuredText
import stix.bindings.data_marking as stix_data_marking_binding

class Marking(stix.Entity):
    _binding = stix_data_marking_binding
    _binding_class = stix_data_marking_binding.MarkingType
    _namespace = 'http://data-marking.mitre.org/Marking-1'

    def __init__(self, markings=None):
        self.markings = markings

    @property
    def markings(self):
        return self._markings

    @markings.setter
    def markings(self, value):
        self._markings = []

        if value is None:
            return
        elif isinstance(value, list):
            for m in value:
                self.add_marking(m)
        else:
            self.add_marking(value)

    def add_marking(self, value):
        if not isinstance(value, MarkingSpecification):
            raise ValueError('value must be instance of MarkingSpecification')
        self._markings.append(value)

    def to_obj(self, return_obj=None, ns_info=None):
        super(Marking, self).to_obj(
            return_obj=return_obj,
            ns_info=ns_info
        )

        obj = self._binding_class()
        obj.Marking = [x.to_obj(ns_info=ns_info) for x in self.markings]
        return obj

    def to_list(self):
        return [x.to_dict() for x in self.markings]

    @staticmethod
    def from_obj(obj):
        if not obj:
            return None

        mlist = [MarkingSpecification.from_obj(x) for x in obj.Marking]
        return Marking(mlist)

    @staticmethod
    def from_list(markings_list):
        if not markings_list:
            return None

        mlist = [MarkingSpecification.from_dict(x) for x in markings_list]
        return Marking(mlist)

    to_dict = to_list
    from_dict = from_list


class MarkingSpecification(stix.Entity):
    _binding = stix_data_marking_binding
    _binding_class = stix_data_marking_binding.MarkingSpecificationType
    _namespace = 'http://data-marking.mitre.org/Marking-1'

    def __init__(self, controlled_structure=None, marking_structures=None):
        self.id_ = None
        self.idref = None
        self.version = None
        self.controlled_structure = controlled_structure

        if isinstance(marking_structures,list):
            self.marking_structures = marking_structures
        elif marking_structures is None:
            self.marking_structures = []
        else:
            raise TypeError('marking_structures must be a list, or None')

        # TODO: add Information_Source

    def to_obj(self, return_obj=None, ns_info=None):
        super(MarkingSpecification, self).to_obj(
            return_obj=return_obj,
            ns_info=ns_info
        )

        obj = self._binding_class()

        obj.id = self.id_
        obj.idref = self.idref
        obj.version = self.version
        obj.Controlled_Structure = self.controlled_structure
        obj.Marking_Structure = [
            x.to_obj(ns_info=ns_info) for x in self.marking_structures
        ]

        return obj

    def to_dict(self):
        d = {}
        if self.id_:
            d['id'] = self.id_
        if self.idref:
            d['idref'] = self.idref
        if self.version:
            d['version'] = self.version
        if self.controlled_structure:
            d['controlled_structure'] = self.controlled_structure
        if self.marking_structures:
            d['marking_structures'] = [
                x.to_dict() for x in self.marking_structures
            ]

        return d

    @staticmethod
    def from_obj(obj):
        if not obj:
            return None

        m = MarkingSpecification()

        m.id_ = obj.id
        m.idref = obj.idref
        m.version = obj.version
        m.controlled_structure = obj.Controlled_Structure
        m.marking_structures = [
            MarkingStructure.from_obj(x) for x in obj.Marking_Structure
        ]

        return m

    @staticmethod
    def from_dict(marking_dict):
        if not marking_dict:
            return None



        get = marking_dict.get  # PEP8 line length fix
        m = MarkingSpecification()

        m.id_ = get('id')
        m.idref = get('idref')
        m.version = get('version')
        m.controlled_structure = get('controlled_structure')
        m.marking_structures = [
            MarkingStructure.from_dict(x) for x in get('marking_structures', [])
        ]

        return m


class MarkingStructure(stix.Entity):
    _binding = stix_data_marking_binding
    _binding_class = stix_data_marking_binding.MarkingStructureType
    _namespace = 'http://data-marking.mitre.org/Marking-1'

    def __init__(self):
        self.marking_model_name = None
        self.marking_model_ref = None

    def to_obj(self, return_obj=None, ns_info=None):
        super(MarkingStructure, self).to_obj(
            return_obj=return_obj,
            ns_info=ns_info
        )

        if not return_obj:
            return_obj = self._binding_class()

        return_obj.marking_model_name = self.marking_model_name
        return_obj.marking_model_ref = self.marking_model_ref

        return return_obj

    def to_dict(self):
        d = {}
        d['xsi:type'] = self._XSI_TYPE
        if self.marking_model_name:
            d['marking_model_name'] = self.marking_model_name
        if self.marking_model_ref:
            d['marking_model_ref'] = self.marking_model_ref

        return d

    @staticmethod
    def lookup_class(xsi_type):
        if not xsi_type:
            raise ValueError("xsi:type is required")
        for (k, v) in _EXTENSION_MAP.iteritems():
            # TODO: for now we ignore the prefix and just check for
            # a partial match
            if xsi_type in k:
                return v

        raise ValueError("Unregistered xsi:type %s" % xsi_type)

    @staticmethod
    def from_obj(obj, partial=None):
        from stix.extensions.marking.tlp import TLPMarkingStructure
        from stix.extensions.marking.simple_marking import SimpleMarkingStructure
        from stix.extensions.marking.terms_of_use_marking import TermsOfUseMarkingStructure
        
        if not obj:
            return None

        if partial:
            m = partial
            m.marking_model_name = obj.marking_model_name
            m.marking_model_ref = obj.marking_model_ref

        else:
            cls = MarkingStructure.lookup_class(obj.xml_type)
            m = cls.from_obj(obj)

        return m

    @staticmethod
    def from_dict(marking_dict, partial=None):
        from stix.extensions.marking.tlp import TLPMarkingStructure
        from stix.extensions.marking.simple_marking import SimpleMarkingStructure
        from stix.extensions.marking.terms_of_use_marking import TermsOfUseMarkingStructure
        
        if not marking_dict:
            return None

        if partial is not None:
            m = partial
            m.marking_model_name = marking_dict.get('marking_model_name')
            m.marking_model_ref = marking_dict.get('marking_model_ref')

        else:
            cls = MarkingStructure.lookup_class(marking_dict.get('xsi:type'))
            m = cls.from_dict(marking_dict)

        return m


#: Mapping of marking extension types to classes
_EXTENSION_MAP = {}
def add_extension(cls):
    _EXTENSION_MAP[cls._XSI_TYPE] = cls




# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

from os.path import abspath, dirname, join
from setuptools import setup, find_packages

INIT_FILE = join(dirname(abspath(__file__)), 'stix', '__init__.py')

def get_version():
    with open(INIT_FILE) as f:
        for line in f.readlines():
            if line.startswith("__version__"):
                version = line.split()[-1].strip('"')
                return version
        raise AttributeError("Package does not have a __version__")

with open('README.rst') as f:
    readme = f.read()

extras_require = {
    'docs': [
        'Sphinx==1.2.1',
        # TODO: remove when updating to Sphinx 1.3, since napoleon will be
        # included as sphinx.ext.napoleon
        'sphinxcontrib-napoleon==0.2.4',
    ],
    'test': [
        'nose==1.3.0',
        'tox==1.6.1',
        'maec>=4.1.0.10,<4.1.1.0'
    ],
}

setup(
    name="stix",
    version=get_version(),
    author="STIX Project, MITRE Corporation",
    author_email="stix@mitre.org",
    description="An API for parsing and generating STIX content.",
    long_description=readme,
    url="http://stix.mitre.org",
    packages=find_packages(),
    install_requires=['lxml>=2.3', 'python-dateutil', 'cybox>=2.1.0.9,<2.1.1.0'],
    extras_require=extras_require,
    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
    ]
)



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix.bindings.ttp as ttp_binding
from stix.common.related import GenericRelationshipList, RelatedTTP


class RelatedTTPs(GenericRelationshipList):
    _namespace = "http://stix.mitre.org/TTP-1"
    _binding = ttp_binding
    _binding_class = _binding.RelatedTTPsType
    _binding_var = "Related_TTP"
    _contained_type = RelatedTTP
    _inner_name = "ttps"



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import unittest
from stix.test import EntityTestCase
from stix.extensions.identity.ciq_identity_3_0 import CIQIdentity3_0Instance


class CIQIdentity3_0InstanceTests(EntityTestCase, unittest.TestCase):
    klass = CIQIdentity3_0Instance
    _full_dict = {'id': 'example:ciqidentity-1',
                  'name': 'John Smith',
                  'roles': ['Programmer', 'Analyst'],
                  'specification': {'party_name': {'name_lines': [{'value': 'Foo'},
                                                                  {'value': 'Bar'}],
                                    'organisation_names': [{'name_elements': [{'element_type': 'FullName',
                                                                               'value': 'Foo Inc.'}],
                                                            'subdivision_names': [{'type': 'Department',
                                                                                   'value': 'InfoSec'}]}],
                                    'person_names': [{'name_elements': [{'value': 'John Smith'}]},
                                                     {'name_elements': [{'value': 'Jill Smith'}]}]}},
                  'xsi:type': 'ciqIdentity:CIQIdentity3.0InstanceType'}


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
import stix.bindings.incident as incident_binding
from .loss_estimation import LossEstimation

class TotalLossEstimation(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding  = incident_binding
    _binding_class = incident_binding.TotalLossEstimationType

    def __init__(self):
        super(TotalLossEstimation, self).__init__()
        self.initial_reported_total_loss_estimation = None
        self.actual_total_loss_estimation = None

    @property
    def initial_reported_total_loss_estimation(self):
        return self._initial_reported_total_loss_estimation

    @initial_reported_total_loss_estimation.setter
    def initial_reported_total_loss_estimation(self, value):
        if not value:
            self._initial_reported_total_loss_estimation = None
        elif isinstance(value, LossEstimation):
            self._initial_reported_total_loss_estimation = value
        else:
            raise ValueError("value must be LossEstimation instance")

    @property
    def actual_total_loss_estimation(self):
        return self._actual_total_loss_estimation

    @actual_total_loss_estimation.setter
    def actual_total_loss_estimation(self, value):
        if not value:
            self._actual_total_loss_estimation = None
        elif isinstance(value, LossEstimation):
            self._actual_total_loss_estimation = value
        else:
            raise ValueError("value must be LossEstimation instance")

    def to_obj(self, return_obj=None, ns_info=None):
        super(TotalLossEstimation, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        obj = self._binding_class()
        if self.initial_reported_total_loss_estimation:
            obj.Initial_Reported_Total_Loss_Estimation = self.initial_reported_total_loss_estimation.to_obj(ns_info=ns_info)
        if self.actual_total_loss_estimation:
            obj.Actual_Total_Loss_Estimation = self.actual_total_loss_estimation.to_obj(ns_info=ns_info)
        return obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.initial_reported_total_loss_estimation = LossEstimation.from_obj(obj.Initial_Reported_Total_Loss_Estimation)
        return_obj.actual_total_loss_estimation = LossEstimation.from_obj(obj.Actual_Total_Loss_Estimation)
        return return_obj

    def to_dict(self):    
        d  = {}
        if self.initial_reported_total_loss_estimation:
            d['initial_reported_total_loss_estimation'] = self.initial_reported_total_loss_estimation.to_dict()
        if self.actual_total_loss_estimation:
            d['actual_total_loss_estimation'] = self.actual_total_loss_estimation.to_dict()
        return d

    @classmethod
    def from_dict(cls, dict_, return_obj=None):
        if not dict_:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.initial_reported_total_loss_estimation = LossEstimation.from_dict(dict_.get('initial_reported_total_loss_estimation'))
        return_obj.actual_total_loss_estimation = LossEstimation.from_dict(dict_.get('actual_total_loss_estimation'))

        return return_obj



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.


import stix
from stix.common import VocabString, StructuredText
import stix.bindings.incident as incident_binding
from stix.common.vocabs import LossProperty, LossDuration
from stix.common.vocabs import AvailabilityLossType as AvailabilityLoss


class NonPublicDataCompromised(VocabString):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding  = incident_binding
    _binding_class = incident_binding.NonPublicDataCompromisedType
    
    def __init__(self, value=None, data_encrypted=None):
        self.data_encrypted = data_encrypted
        super(NonPublicDataCompromised, self).__init__(value)
    
    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()
        
        super(NonPublicDataCompromised, cls).from_obj(obj, return_obj=return_obj)
        return_obj.data_encrypted = obj.data_encrypted
        return return_obj
    
    def to_obj(self, return_obj=None, ns_info=None):
        if not return_obj:
            return_obj = self._binding_class()
        
        super(NonPublicDataCompromised, self).to_obj(return_obj=return_obj, ns_info=ns_info)
        return_obj.data_encrypted = self.data_encrypted
        return return_obj
    
    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()
            
        super(NonPublicDataCompromised, cls).from_dict(d, return_obj=return_obj)
        return_obj.data_encrypted = d.get('data_encrypted')
        return return_obj
    
    def to_dict(self):
        d = super(NonPublicDataCompromised, self).to_dict()
        if self.data_encrypted:
            d['data_encrypted'] = self.data_encrypted
        return d

class PropertyAffected(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding = incident_binding
    _binding_class = incident_binding.PropertyAffectedType
    
    def __init__(self):
        self.property_ = None
        self.description_of_effect = None
        self.type_of_availability_loss = None
        self.duration_of_availability_loss = None
        self.non_public_data_compromised = None
        
    @property
    def property_(self):
        return self._property
    
    @property_.setter
    def property_(self, value):
        if not value:
            self._property = None
        elif isinstance(value, VocabString):
            self._property = value
        else:
            self._property = LossProperty(value)
        
    @property
    def description_of_effect(self):
        return self._description_of_effect
    
    @description_of_effect.setter
    def description_of_effect(self, value):
        if not value:
            self._description_of_effect = None
        elif isinstance(value, StructuredText):
            self._description_of_effect = value
        else:
            self._description_of_effect = StructuredText(value)

    @property
    def type_of_availability_loss(self):
        return self._type_of_availability_loss
    
    @type_of_availability_loss.setter
    def type_of_availability_loss(self, value):
        if not value:
            self._type_of_availability_loss = None
        elif isinstance(value, VocabString):
            self._type_of_availability_loss = value
        else:
            self._type_of_availability_loss = AvailabilityLoss(value)
            
    @property
    def duration_of_availability_loss(self):
        return self._duration_of_availability_loss
    
    @duration_of_availability_loss.setter
    def duration_of_availability_loss(self, value):
        if not value:
            self._duration_of_availability_loss = None
        elif isinstance(value, VocabString):
            self._duration_of_availability_loss = value
        else:
            self._duration_of_availability_loss = LossDuration(value)
    
    @property
    def non_public_data_compromised(self):
        return self._non_public_data_compromised
    
    @non_public_data_compromised.setter
    def non_public_data_compromised(self, value):
        if not value:
            self._non_public_data_compromised = None
        elif isinstance(value, NonPublicDataCompromised):
            self._non_public_data_compromised = value
        else:
            self._non_public_data_compromised = NonPublicDataCompromised(value)
    
    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None
        if not return_obj:
            return_obj = cls()
            
        return_obj.property_ = VocabString.from_obj(obj.Property)
        return_obj.description_of_effect = StructuredText.from_obj(obj.Description_Of_Effect)
        return_obj.type_of_availability_loss = VocabString.from_obj(obj.Type_Of_Availability_Loss)
        return_obj.duration_of_availability_loss = VocabString.from_obj(obj.Duration_Of_Availability_Loss)
        return_obj.non_public_data_compromised = NonPublicDataCompromised.from_obj(obj.Non_Public_Data_Compromised)
        return return_obj
    
    def to_obj(self, return_obj=None, ns_info=None):
        super(PropertyAffected, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        if not return_obj:
            return_obj = self._binding_class()
            
        if self.property_:
            return_obj.Property = self.property_.to_obj(ns_info=ns_info)
        if self.description_of_effect:
            return_obj.Description_Of_Effect = self.description_of_effect.to_obj(ns_info=ns_info)
        if self.type_of_availability_loss:
            return_obj.Type_Of_Availability_Loss = self.type_of_availability_loss.to_obj(ns_info=ns_info)
        if self.duration_of_availability_loss:
            return_obj.Duration_Of_Availability_Loss = self.duration_of_availability_loss.to_obj(ns_info=ns_info)
        if self.non_public_data_compromised:
            return_obj.Non_Public_Data_Compromised = self.non_public_data_compromised.to_obj(ns_info=ns_info)
        
        return return_obj
    
    @classmethod
    def from_dict(cls, d, return_obj=None):
        if not d:
            return None
        if not return_obj:
            return_obj = cls()
            
        return_obj.property_ = VocabString.from_dict(d.get(('property')))
        return_obj.description_of_effect = StructuredText.from_dict(d.get('description_of_effect'))
        return_obj.type_of_availability_loss = VocabString.from_dict(d.get('type_of_availability_loss'))
        return_obj.duration_of_availability_loss = VocabString.from_dict(d.get('duration_of_availability_loss'))
        return_obj.non_public_data_compromised = NonPublicDataCompromised.from_dict(d.get('non_public_data_compromised'))
        
        return return_obj
    
    def to_dict(self):
        d = {}
        if self.property_:
            d['property'] = self.property_.to_dict()
        if self.description_of_effect:
            d['description_of_effect'] = self.description_of_effect.to_dict()
        if self.type_of_availability_loss:
            d['type_of_availability_loss'] = self.type_of_availability_loss.to_dict()
        if self.duration_of_availability_loss:
            d['duration_of_availability_loss'] = self.duration_of_availability_loss.to_dict()
        if self.non_public_data_compromised:
            d['non_public_data_compromised'] = self.non_public_data_compromised.to_dict()
        return d
    



import unittest

from stix.test import EntityTestCase

from stix.common import InformationSource


class InformationSourceTests(EntityTestCase, unittest.TestCase):
    klass = InformationSource
    _full_dict = {
        'description': "An amazing source",
        'identity': {
            'name': "Spiderman",
        },
        'references' : ['http://example.com'],
        'time': {
            'start_time': "2010-11-12T01:02:03",
            'end_time': "2013-12-11T03:02:01",
        },
        'tools': [
            {
                'name': "Web",
                'description': "Superwebs",
            },
        ],
        'roles': [
            {
                'value': 'Initial Author',
                'xsi:type': 'stixVocabs:InformationSourceRoleVocab-1.0'
            },
            {
                'value': 'Transformer/Translator',
                'xsi:type': 'stixVocabs:InformationSourceRoleVocab-1.0'
            }
        ]
    }


if __name__ == "__main__":
    unittest.main()



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import stix
from stix.common import VocabString
from stix.common.vocabs import ImpactRating
import stix.bindings.incident as incident_binding

class DirectImpactSummary(stix.Entity):
    _namespace = "http://stix.mitre.org/Incident-1"
    _binding  = incident_binding
    _binding_class = incident_binding.DirectImpactSummaryType

    def __init__(self):
        super(DirectImpactSummary, self).__init__()
        self.asset_losses = None
        self.business_mission_disruption = None
        self.response_and_recovery_costs = None

    @property
    def asset_losses(self):
        return self._asset_losses

    @asset_losses.setter
    def asset_losses(self, value):
        if not value:
            self._asset_losses = None
        elif isinstance(value, VocabString):
            self._asset_losses = value
        else:
            self._asset_losses = ImpactRating(value=value)

    @property
    def business_mission_disruption(self):
        return self._business_mission_disruption

    @business_mission_disruption.setter
    def business_mission_disruption(self, value):
        if not value:
            self._business_mission_disruption = None
        elif isinstance(value, VocabString):
            self._business_mission_disruption = value
        else:
            self._business_mission_disruption = ImpactRating(value=value)

    @property
    def response_and_recovery_costs(self):
        return self._response_and_recovery_costs

    @response_and_recovery_costs.setter
    def response_and_recovery_costs(self, value):
        if not value:
            self._response_and_recovery_costs = None
        elif isinstance(value, VocabString):
            self._response_and_recovery_costs = value
        else:
            self._response_and_recovery_costs = ImpactRating(value=value)
            
    def to_obj(self, return_obj=None, ns_info=None):
        super(DirectImpactSummary, self).to_obj(return_obj=return_obj, ns_info=ns_info)

        obj = self._binding_class()
        if self.asset_losses:
            obj.Asset_Losses = self.asset_losses.to_obj(ns_info=ns_info)
        if self.business_mission_disruption:
            obj.Business_Mission_Disruption = self.business_mission_disruption.to_obj(ns_info=ns_info)
        if self.response_and_recovery_costs:
            obj.Response_And_Recovery_Costs = self.response_and_recovery_costs.to_obj(ns_info=ns_info)
        return obj

    @classmethod
    def from_obj(cls, obj, return_obj=None):
        if not obj:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.asset_losses = VocabString.from_obj(obj.Asset_Losses)
        return_obj.business_mission_disruption = VocabString.from_obj(obj.Business_Mission_Disruption)
        return_obj.response_and_recovery_costs = VocabString.from_obj(obj.Response_And_Recovery_Costs)
        return return_obj

    def to_dict(self):    
        d  = {}
        if self.asset_losses:
            d['asset_losses'] = self.asset_losses.to_dict()
        if self.business_mission_disruption:
            d['business_mission_disruption'] = self.business_mission_disruption.to_dict()
        if self.response_and_recovery_costs:
            d['response_and_recovery_costs'] = self.response_and_recovery_costs.to_dict()
        return d

    @classmethod
    def from_dict(cls, dict_, return_obj=None):
        if not dict_:
            return None

        if not return_obj:
            return_obj = cls()

        return_obj.asset_losses = VocabString.from_dict(dict_.get('asset_losses'))
        return_obj.business_mission_disruption = VocabString.from_dict(dict_.get('business_mission_disruption'))
        return_obj.response_and_recovery_costs = VocabString.from_dict(dict_.get('response_and_recovery_costs'))

        return return_obj



# Copyright (c) 2014, The MITRE Corporation. All rights reserved.
# See LICENSE.txt for complete terms.

import uuid

#: Default "example" namespace used for identifiers when no other namespace is
#: defined.
EXAMPLE_NAMESPACE = {'http://example.com' : 'example'}

class InvalidMethodError(ValueError):
    def __init__(self, method):
        ValueError.__init__(self, "invalid method: %s" % method)


class IDGenerator(object):
    """Utility class for generating STIX ids"""
    METHOD_UUID = 1
    METHOD_INT = 2

    METHODS = (METHOD_UUID, METHOD_INT,)

    def __init__(self, namespace=None, method=METHOD_UUID):
        self.namespace = namespace or EXAMPLE_NAMESPACE
        self.method = method
        self.next_int = 1

    @property
    def namespace(self):
        return self._namespace

    @namespace.setter
    def namespace(self, value):
        if not isinstance(value, dict):
            raise ValueError("Must be a dictionary: ex {'http://example.com' : 'example'}")

        if len(value) != 1:
            raise ValueError("Provided dictionary must have at most one entry")

        self._namespace = value

    @property
    def method(self):
        return self._method

    @method.setter
    def method(self, value):
        if value not in IDGenerator.METHODS:
            raise InvalidMethodError("invalid method: %s" % value)
        self._method = value

    def create_id(self, prefix="guid"):
        """Create an ID.

        Note that if `prefix` is not provided, it will be `quid`, even if the
        `method` is `METHOD_INT`.
        """
        if self.method == IDGenerator.METHOD_UUID:
            id_ = str(uuid.uuid4())
        elif self.method == IDGenerator.METHOD_INT:
            id_ = self.next_int
            self.next_int += 1
        else:
            raise InvalidMethodError()

        _, ns_prefix = self.namespace.iteritems().next()
        return "%s:%s-%s" % (ns_prefix, prefix, id_)



#: Singleton instance within this module. It is lazily instantiated, so simply
#: importing the utils module will not create the object.
__generator = None


def _get_generator():
    """Return the `stix.utils` module's generator object.

    Only under rare circumstances should this function be called by external
    code. More likely, external code should initialize its own IDGenerator or
    use the `set_id_namespace`, `set_id_method`, or `create_id` functions of
    the `stix.utils` module.
    """
    global __generator
    if not __generator:
        __generator = IDGenerator()
    return __generator


def set_id_namespace(namespace):
    """ Set the namespace for the module-level ID Generator"""
    _get_generator().namespace = namespace

def set_id_method(method):
    """ Set the method for the module-level ID Generator"""
    _get_generator().method = method

def get_id_namespace():
    """Return the namespace associated with generated ids"""
    return _get_generator().namespace.iterkeys().next()

def get_id_namespace_alias():
    """Returns the namespace alias assoicated with generated ids"""
    return _get_generator().namespace.itervalues().next()

def create_id(prefix=None):
    """ Create an ID using the module-level ID Generator"""
    if not prefix:
        return _get_generator().create_id()
    else:
        return _get_generator().create_id(prefix)
